%> @file PlcCharArray.m
% This file is protected by TIS
% Copyright  2013-2022.

% PlcCharArray
%>   Represents a PLC array which addresses an unique data entry stored within a programmable logic controller (PLC) its values are interpreted as character values.
classdef PlcCharArray < PlcArray
    % ---------- Public properties ----------
    properties (Access = public, Dependent = true)
        %> @brief Gets or sets the value assigned to the PLC value.
        %>   The value assigned to the PLC value.
        Value;
    end % properties

    % ---------- Public constructors ----------
    methods (Access = public)
        %> @brief Initializes a new instance of the PlcCharArray class using the specified @b address and @b valuesOrLength.
        %>
        %> @param address The PlcAddress of the new PlcCharArray to which the
        %>                PLC values belong.
        %> @param valuesOrLength
        %>        * @b values: The initial values of the new PlcCharArray.
        %>        * @b length: The array length used to store the array values of the PlcCharArray.
        %>
        %> @b Errors
        %> * The @b address does not refer to the @b PlcRawType.Byte.
        %> * The @b length is out of the bounds defined by @b PlcType.MinLength
        %>   and @b PlcType.MaxLength.
        function this = PlcCharArray(address, valuesOrLength)
            if (iscell(valuesOrLength))
                values = NET.createArray('System.Char', length(valuesOrLength));

                for index = 1:length(valuesOrLength)
                    values(index) = System.Convert.ToChar(valuesOrLength{index});
                end
            elseif (length(valuesOrLength) > 1)
                values = NET.createArray('System.Char', length(valuesOrLength));

                for index = 1:length(valuesOrLength)
                   values(index) = System.Convert.ToChar(valuesOrLength(index));
                end
            else
                values = NET.createArray('System.Char', valuesOrLength);

                % Initialize new array with blanks; otherwise MatLab is not able to work
                % with the .NET array.
                for index = 1:valuesOrLength
                   values(index) = ' ';
                end
            end

            this = this@PlcArray( ...
                    PlcTypeCode.Char, ...
                    address, ...
                    values);
        end
    end % methods
    
    % ---------- Property accessors ----------
    methods
        %> Gets the value assigned to the PLC value.
        function value = get.Value(this)
            value = char.empty(this.Length, 0);
            managedValue = this.ManagedInstance.GetValue('Value');

            for index = 1:this.Length
                value(index) = managedValue(index);
            end
        end

        %> Sets the value assigned to the PLC value.
        function set.Value(this, value)
            managedValue = this.ManagedInstance.GetValue('Value');

            if (iscell(value))
                for index = 1:min(length(value), this.Length)
                    managedValue.Set(index - 1, System.Convert.ToChar(value{index}));
                end
            elseif (length(value) > 1)
                for index = 1:min(length(value), this.Length)
                    managedValue.Set(index - 1, System.Convert.ToChar(value(index)));
                end
            end

            this.ManagedInstance.SetValue('Value', managedValue);
        end
    end
end

